#include <stdio.h>
#include <memory.h>

static const size_t DOS_NAME_MAX = (8+1+3);

static void usage() {
	printf("usgae: divsyn [filename]\n");
}

static inline unsigned int getdwordle(unsigned char *src) {
	return src[0] + (src[1] << 8) + (src[2] << 16) + (src[3] << 24);
}

int main(int argc, char *argv[]) {

	if (argc < 2) {
		usage();
		return -1;
	}

	FILE *src(NULL), *dst(NULL);
	unsigned char *buf(NULL);
	try {
		src = fopen(argv[1], "rb");
		if (src == NULL)
		  throw -2;

		unsigned char size_buf[4];
		if (fread(size_buf, 1, 2, src) < 2)
		  throw -3;

		char name_buf[DOS_NAME_MAX+1] = {0};
		unsigned int file_num = size_buf[0] + (size_buf[1] << 8);
		unsigned int data_start = 2 + (file_num+1) * 16;
		unsigned int ofs_start(0), ofs_end(0), file_size(0);

		for (unsigned int i=0; i < file_num; i++) {
			fread(name_buf, 1, DOS_NAME_MAX, src);
			name_buf[DOS_NAME_MAX] = 0;

			memset(size_buf, 0, 4);
			if (fread(size_buf, 1, 4, src) < 4)
			  throw -3;
			ofs_start = getdwordle(size_buf);

			fseek(src, 12, SEEK_CUR);
			memset(size_buf, 0, 4);
			if (fread(size_buf, 1, 4, src) < 4)
			  throw -3;
			ofs_end = getdwordle(size_buf);
			file_size = ofs_end - ofs_start;
			fseek(src, -16, SEEK_CUR);

			printf("name: %12s offset: %5d size: %5d\n", name_buf, ofs_start, file_size);

			dst = fopen(name_buf, "wb");
			if (dst) {
				buf = new unsigned char[file_size];

				fseek(src, data_start + ofs_start, SEEK_SET);
				fread(buf, 1, file_size, src);
				fwrite(buf, 1, file_size, dst);
				delete [] buf;
				buf = NULL;
				fclose(dst);
				dst = NULL;
			}
			fseek(src, 2 + (i+1) * 16, SEEK_SET);
		}
		fclose(src);
		src = NULL;
	} catch (int ret) {
		if (src) fclose(src);
		if (dst) fclose(dst);
		if (buf) delete [] buf;
		switch(ret) {
		  case -2:
			printf("open error\n");
			break;
		  case -3:
			printf("read error\n");
			break;
		  default:
			printf("error\n");
		}
	} catch (...) {
		printf("error\n");
	}

	return 0;
}

